require 'forwardable'

module Take
  class Project
    class File

      extend Forwardable

      def self.normalize(file, path)
        expanded = ::File.expand_path(file, path)
        raise ArgumentError, "The given file cannot be placed in " \
          "the project: #{file.inspect}" \
          unless expanded.start_with?(path.to_s)
        name = Pathname.new(expanded)
        name.relative_path_from(path)
      end

      def_delegators :@name, :to_path, :to_s, :extname

      def initialize(name)
        @name = name
      end

      def type
        @_type ||= begin
          case @name.extname
          when ".ta"
            :library
          when ".to", ".o"
            :object
          when ".c"
            :source
          when ""
            :executable
          else
            :unknown
          end
        end
      end


      def sub_ext(ext)
        File.new(@name.sub_ext(ext))
      end
    end
  end
end
